function [outparv,lnjac,lnprior]=DSGE_repar2(inparv,R,direction,prior_type_ndxv,nmv)
% PURPOSE: performs reparameterisation and compute lnjac and lnprior (kernel)
% in DSGE model 
% differs from previous version because it allows Dirichlet distribution
% (multiple chains with m_i potentially different cardinalities. 
% -----------------------------------------------------
% USAGE: 
% where: 
% add documentation
% -----------------------------------------------------
% NOTE: no dimension checks imposed , 2008_05_26
% differs from AT1 counterpart in allowing for Inverse Gamma distributed
% parameters
% 2010/03/12 added uniform between [a,b]
% 2022/02/08 added Dirichlet (need a further argument (nmv, the vector with
% cardinalities of each chain)
% -----------------------------------------------------
% RETURNS: 
%         outparv = (npar x 1) parameter vector
%         lnjac = ln jacobian 
%         lnprior = ln prior (kernel only)
% -----------------------------------------------------
% written by:
% Gianni Amisano, Dept of Economics
% University of Brescia
% amisano@eco.unibs.it
npar=size(inparv,1);
outparv=zeros(npar,1);
lnjac=zeros(npar,1);
lnprior=zeros(npar,1);
if direction~=0
    %direct transformation from omega to delta (from unbounded to bounded)
    % Beta distributed parameters
    ndx=find(prior_type_ndxv==1);
    outparv(ndx)=(1+exp(-(inparv(ndx)))).^(-1);
    p1=log(outparv(ndx));%ln(delta)
    p2=log(1-outparv(ndx)); %ln(1-delta)
    lnjac(ndx)=-(p1+p2);
    lnprior(ndx)=((R(ndx,1)-1).*p1 +(R(ndx,2)-1).*p2);
    % Gamma distributed parameters
    ndx=find(prior_type_ndxv==2);
    p1=inparv(ndx);%omega
    p2=exp(p1);%delta
    outparv(ndx)=p2;
    lnjac(ndx)=-p1;
    lnprior(ndx)=((R(ndx,2)/2-1).*p1-R(ndx,1).*p2/2);
    % shifted Gamma distributed parameters
    ndx=find(prior_type_ndxv==3);
    p1=inparv(ndx);%omega
    p2=exp(p1);%delta-1
    outparv(ndx)=p2+1;
    lnjac(ndx)=-p1;
    lnprior(ndx)=((R(ndx,2)/2-1).*p1-R(ndx,1).*(p2)/2);    
    % Inverse Gamma distributed parameters
    ndx=find(prior_type_ndxv==5);
    p1=inparv(ndx);%omega
    p2=exp(p1);%delta
    outparv(ndx)=p2;
    lnjac(ndx)=-p1;
    lnprior(ndx)=-((R(ndx,2)+1).*p1+((R(ndx,1).^2).*R(ndx,2))./(2*(p2.^2)));
    % uniform between a and b
    ndx=find(prior_type_ndxv==6);
    av=min(R(ndx,:),[],2);
    bv=max(R(ndx,:),[],2);
    p1=inparv(ndx);%omega
    p2=(av+bv.*exp(p1))./(1+exp(p1));%delta
    outparv(ndx)=p2;
    lnjac(ndx)=log(bv-av)-log(p2-av)-log(bv-p2);
    % Dirichlet prior
    ndx=find(prior_type_ndxv==7);
    if numel(ndx)
        nc=numel(nmv);
        icum=0;
        for ic=1:nc
            nmi=nmv(ic);
            for im=1:nmi
                % construct the vector of hyperparameters (rem that on first
                % col we read hyperpar on prob of staying on state and on second column we read the
                % hyperpar on probs to leave the state
                rv=R(ndx(icum+1),2)*ones(nmi,1);
                rv(im)=R(ndx(icum+1),1);
                % repar
                p1=inparv(ndx((icum+1):(icum+nmi-1))); %omega
                [p2,lnjac2] = dirichlet_repar2(p1,1);   %delta
                outparv(ndx((icum+1):(icum+nmi-1)))=p2;
                lnjac(ndx((icum+1):(icum+nmi-1)))=lnjac2/(nmi-1);
                lnprior(ndx((icum+1):(icum+nmi-1)))=sum((rv-1).*log([p2; 1-sum(p2)]))/(nmi-1);
                icum=icum+nmi-1;
            end
        end
    end
 else
    %INVERSE transformation from delta to omega (from bounded to unbounded)
    % Beta distributed parameters
    ndx=find(prior_type_ndxv==1);
    p1=log(inparv(ndx));%ln(delta)
    p2=log(1-inparv(ndx));%ln(1-delta)
    outparv(ndx)=p1-p2;
    lnjac(ndx)=(p1+p2);
    lnprior(ndx)=((R(ndx,1)-1).*p1 +(R(ndx,2)-1).*p2);
    % Gamma distributed parameters
    ndx=find(prior_type_ndxv==2);
    p1=inparv(ndx);%delta
    p2=log(p1);%omega
    outparv(ndx)=p2;
    lnjac(ndx)=p2;
    lnprior(ndx)=((R(ndx,2)/2-1).*p2-R(ndx,1).*p1/2);
    % shifted gamma distributed parameters
    ndx=find(prior_type_ndxv==3);
    p1=inparv(ndx)-1;%delta-1
    p2=log(p1);%omega
    outparv(ndx)=p2;
    lnjac(ndx)=p2;    
    lnprior(ndx)=((R(ndx,2)/2-1).*p2-R(ndx,1).*(p1)/2);
    % Inverse Gamma distributed parameters
    ndx=find(prior_type_ndxv==5);
    p1=inparv(ndx);%delta
    p2=log(p1);%omega
    outparv(ndx)=p2;
    lnjac(ndx)=p2;
    lnprior(ndx)=-((R(ndx,2)+1).*p2+((R(ndx,1).^2).*R(ndx,2))./(2*(p1.^2)));    
% uniform between a and b
    ndx=find(prior_type_ndxv==6);
    av=min(R(ndx,:),[],2);
    bv=max(R(ndx,:),[],2);
    p1=inparv(ndx);%delta
    p2=log((p1-av)./(bv-p1));%omega
    outparv(ndx)=p2;
    lnjac(ndx)=-log(bv-av)+log(p1-av)+log(bv-p1);
    
    % Dirichlet prior
    ndx=find(prior_type_ndxv==7);
    if numel(ndx)
        nc=numel(nmv);
        icum=0;
        for ic=1:nc
            nmi=nmv(ic);
            for im=1:nmi
                % construct the vector of hyperparameters (rem that on first
                % col we read hyperpar on prob of staying on state and on second column we read the
                % hyperpar on probs to leave the state
                rv=R(ndx(icum+1),2)*ones(nmi,1);
                rv(im)=R(ndx(icum+1),1);
                % repar
                p2=inparv(ndx((icum+1):(icum+nmi-1))); %delta
                [p1,lnjac2] = dirichlet_repar2(p2,0);   %omega
                outparv(ndx((icum+1):(icum+nmi-1)))=p1;
                lnjac(ndx((icum+1):(icum+nmi-1)))=lnjac2/(nmi-1);
                lnprior(ndx((icum+1):(icum+nmi-1)))=sum((rv-1).*log([p2; 1-sum(p2)]))/(nmi-1);
                icum=icum+nmi-1;
            end
        end
    end
    
    
    
end
% Gaussian distributed parameters (here no transformation is involved)
% therefore the code is the same in the direct and inverse transformation
% and lnjac=0
ndx=find(prior_type_ndxv==4);
p1=inparv(ndx);%omega=delta
outparv(ndx)=p1;
lnprior(ndx)=-(((p1-R(ndx,1))./R(ndx,2)).^2)/2;