%
% Function: solab1
%
% Purpose: Solves for the recursive representation of the stable solution to a system
% of linear difference equations.
% differs from solab in returning a code and dummy output s(f=p=0)
% if invertibility conditions not met
% Modification by Gianni Amisano, 09/08/05

% Inputs: Two square matrices a and b and a natural number nk
%
% a and b are the coefficient matrices of the difference equation
%
% a*x(t+1) = b*x(t)
% 
% where x(t) is arranged so that the state variables come first, and
%
% nk is the number of state variables.
%
% Outputs: the decision rule f and the law of motion p AND A RETURN CODE. 
% IF RETURN CODE = 0, INVERTIBILITY CONDITIONS HAVE BEEN VIOLATED
% If we write
%
% x(t) = [k(t);u(t)] where k(t) contains precisely the state variables, then
% 
% u(t)   = f*k(t) and
%
% k(t+1) = p*k(t).
%
% Calls: qzdiv
%

function [retcode,f,p] = solab1(a,b,nk)
test=(sum(sum(isfinite(a)))==numel(a))*(sum(sum(isfinite(b)))==numel(b));
if test==0
    retcode=0;
    f=zeros(size(a,1)-nk,nk);
    p=zeros(nk,nk);
else
    retcode=1;
    [s,t,q,z] = qz(a,b);            % upper triangular factorization of the matrix pencil b-za
    [s,t,q,z] = qzdiv(1,s,t,q,z);   % reordering of generalized eigenvalues in ascending order

    z21 = z(nk+1:end,1:nk);
    z11 = z(1:nk,1:nk);

    %rank(z11)
    %size(z11)
    %nk
    test=sum(sum(isfinite(z11)))==numel(z11);
    if test==0
        retcode=0;
        f=0*z21;
        p=0*z11;
    else
        if rank(z11)<nk;
            %error('Invertibility condition violated')
            retcode=0;
            f=0*z21;
            p=0*z11;
            %end
        else
            z11i = z11\eye(nk);
            s11 = s(1:nk,1:nk);
            t11 = t(1:nk,1:nk);

            if abs(t(nk,nk))>abs(s(nk,nk)) | abs(t(nk+1,nk+1))<abs(s(nk+1,nk+1));
                retcode=0;
                %warning('Wrong number of stable eigenvalues.');
            end

            if rcond(s11) > 1e-12
                dyn = s11\t11;
                f = real(z21*z11i);
                p = real(z11*dyn*z11i);
            else
                retcode=0;
                f=0*z21;
                p=0*z11;
            end
        end
    end
end