% Function: solab2_new_ms_retcode
% Written by Gianni Amisano and Oreste Tristani in February 2013
% differs from solab2_new_ms in returning a retcode
%
% Extension of the Gomme and Klein code solab2 to the case with regime
%           switching shocks
% The code also uses a sylvester3.m from Dynare to speed up the solution
%
% Purpose: Solves for the second-order accurate approximation
%          of the solution to a dynamic model
%
% The model is E_t[f(x_{t+1},y_{t+1},x_t,y_t)]=0
%
% Inputs: gra, an (nx+ny) by 2*(nx+ny) matrix (the gradient of f)
%         hes, an 2*(nx+ny)^2 by 2*(nx+ny) matrix (the Hessian of f)
%         ssigma, an nx by nx by ns object; the variance of
%           eps_{t+1}:=x_{t+1}-E_t[x_{t+1}] in the different states ns
%         nx, a natural number
%
% For the definition of the gradient and the Hessian, see
% Magnus and Neudecker: Matrix Differential Calculus with
% Applications in Statistics and Econometrics.
%
% Outputs: the matrices ff, pp, ee, gg, kx_s and ky_s
%          whose roles are described below
%
% x_{t+1} = kx_s + Px_t + (1/2)*kron(eye(nx),x'_t)Gx_t + eps_{t+1}
%
% y_{t}   = ky_s + Fx_t + (1/2)*kron(eye(ny),x'_t)Ex_t
%
% Calls: solab, tracem, sylvester3
%

function [retcode,ff,pp,ee,gg,kx_s,ky_s] = solab2_new_ms_retcode(tr_prob,gra,hes,ssigma,nx)
retcode1=1;
m = size(gra,1);   % Number of variables
ny = m-nx;			 % Number of non-state variables
%
ns = size(ssigma,3);
%
aa = gra(:,1:m);
bb = -gra(:,m+1:end);
[retcode,ff,pp] = solab1(aa,bb,nx);  % Linear approximation
%
if retcode~=0
    f1 = gra(:,1:nx);
    f2 = gra(:,nx+1:m);
    f4 = gra(:,m+nx+1:2*m);
    
    mm = [pp;ff*pp;eye(nx);ff];
    aa1 = kron(eye(m),mm')*hes*mm;
    bb1 = kron(f1,eye(nx));
    bb2 = kron(f2,eye(nx));
    bb4 = kron(f4,eye(nx));
    cc1 = kron(eye(ny),pp');
    cc2 = kron(ff,eye(nx));
    
    % aa = [kron(eye(nx),bb4)+kron(pp',bb2*cc1) kron(eye(nx),bb1+bb2*cc2)];
    % sol = -aa\aa1(:);
    % ee = reshape(sol(1:nx^2*ny),nx*ny,nx);
    % gg = reshape(sol(nx^2*ny+1:end),nx^2,nx);
    
    % =====================================
    % sylvester3.m is from dynare
    sol = sylvester3([bb4,bb1 + bb2*cc2],[bb2*cc1,zeros((nx+ny)*nx,nx^2)],pp,-aa1);
    
    ee = sol(1:nx*ny,:);
    gg = sol(nx*ny+1:nx*ny+nx^2,:);
    % =====================================
    
    ma = 2*[kron(eye(ns),f1+f2*ff) kron(tr_prob',f2)+kron(eye(ns),f4)];    % Corrected in Feb 2013
    if abs(rcond(ma)) < eps
        retcode1=0;
        ff=zeros(ny,nx);
        pp=zeros(nx,nx);
        ee=zeros(nx*ny,nx);
        gg=zeros(nx^2,nx);    
        kx_s=zeros(nx,ns);
        ky_s=zeros(ny,ns);
    else
        eyeff = [eye(nx);ff;zeros(m,nx)];
        ve = zeros(nx+ny,ns) ;
        for i = 1:ns
            ssigma_st = squeeze(ssigma(:,:,i));
            ve(:,i) = f2*tracem(kron(eye(ny),ssigma_st)*ee)+...
                tracem(kron(eye(m),eyeff')*hes*eyeff*ssigma_st);
        end
        kxy = -ma\vec(ve);
        kx_s = reshape(kxy(1:nx*ns,:),nx,ns);
        ky_s = reshape(kxy(nx*ns+1:end,:),ny,ns);
    end
else
    kx_s=zeros(nx,ns);
    ky_s=zeros(ny,ns);
    ff=zeros(ny,nx);
    pp=zeros(nx,nx);
    ee=zeros(nx*ny,nx);
    gg=zeros(nx^2,nx);
end
%   
retcode=retcode*retcode1;